#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <ctype.h>
#include <unistd.h>
#define DEBUG 0
#define CUTOFF .9
#define numtrials 10

//        Only use 90% of the data in  Residual


float NCurve(float s, float B, float n1, float sigma) {
	return exp( B - n1* s - sigma*s*s );
}

inline float pabs( float x){ return sqrt(x*x);}

struct MicScop{		float AccVolt; float Cs;
				float Pre ;float quad ; float linear;
				float defocus ;float phi ; float Amp; float Bfactor; };


//        
float Residual(float *sData, float *IntData,float B, float n1, float sigma, int NumPoints, int Ismin) {
	float Res2=0, ResTemp;
	float Curve, s;
	int icount=0;
	int NumPointsCut = NumPoints*CUTOFF;

	for (int i=Ismin; i < NumPointsCut ; i++){
		if (sigma<0) { Res2 = -1; break;}
		s=sData[i];
		Curve = NCurve(s,B,n1,sigma);
		ResTemp  = IntData[i] - Curve;     // Want data to have positive scattering
		if (ResTemp < 0) { Res2 = -1; i= NumPoints;} 
		else { 
			icount++;
			Res2 += ResTemp*ResTemp;
//		Res2 += sqrt(ResTemp);
//		Res2 += (ResTemp/Curve*Curve);
		}
	}

	
	return Res2;
}

/****************************************************************/

void printCurve(float *sData, float *Data, int NumPoints,int Ismin, FILE* Curve) {

	for (int i=Ismin; i < NumPoints; i++){
//		printf("%4.4e	%4.4e \n",sData[i],Data[i]	);
		fprintf(Curve,"%4.4e	%4.4e \n",sData[i],Data[i]	);
	}	
	return;
}

float ResidPredVsObs(int pSFMax,int NumPoints,float *sData, float *StructFactData,float *ResDatak,MicScop MicAllk,int type) {
	float MicEffect ;
	float ResidPred, Difference, TotalDiff = 0;
	float s5,Pi=3.1415926;
	float gamma, epsilon, lambda_s, lambda_s2;
	float V = MicAllk.AccVolt;  // eV
	float hc = 12400; // eV-A
	float E0 = 511*1000; // eV
	float lambda =  hc / (sqrt(V*V + 2*E0*V));  // That's the wavelength
	for (int p5=0; p5 < NumPoints; p5++) {
		s5= sData[p5] ;
		lambda_s = lambda *s5;
		lambda_s2= lambda_s*lambda_s;
		if (s5 <.3 && p5> pSFMax+3) {    //       %% if  close to the origin and p5  past the max      
					gamma = -2*Pi*( MicAllk.defocus*lambda_s2/2 + MicAllk.Cs* lambda_s2*lambda_s2/4)/ lambda;
					MicEffect =  MicAllk.Amp* exp(-MicAllk.Bfactor*s5*s5);
					MicEffect *= sin(gamma+MicAllk.phi)*sin(gamma+MicAllk.phi);
					MicEffect += epsilon;
					ResidPred   = MicEffect* StructFactData[p5];
					if (type==0) Difference  = ResidPred - ResDatak[p5];
					if (type==1) Difference  = (ResidPred - ResDatak[p5])/(sqrt( ResidPred + ResDatak[p5] )) ;
					TotalDiff += Difference*Difference;
				}
			}
	return TotalDiff;
}

/****************************************************************/


	
void FitNoise(int NumPoints, int Ismin, float * sData, float *IntData, float *B9, float *n19, float *sigma9, int *IFirstZero9) {

//      %%  1.  Now we fit the noise part:         the amplitude, exp decay, and Gaussian decay  
//       %% Make Two passes

	float ResTemp,  Res2, ResMin;
	int IFirstZero, iMin,iPass; 
	float sigmaBest, n1Best, BBest;
	float sigmaBestNew, n1BestNew, BBestNew;
	float sigmaMax;
	float B, Bmin, sigma, n1;
	int NumPointsCut = NumPoints*CUTOFF;
	float tailLogSlope = (log ((IntData[NumPointsCut-1]) /(IntData[NumPointsCut-4] )) )/( sData[NumPointsCut-1] - sData[NumPointsCut-4]);
	float residual, MaxPeakVal;





//                0. Set it Up
//      0A. Get the slope at infinity
//  	 g = ln f = B - n1 s - sigma s^2  =  V //
//    So   dg/ds =  - n1  - 2*sigma  s   =       T//

//      This system is solved by   
//               n1 = -T - 2*sigma s
//			      B  = V -Ts - sigma s^2  
//
//     0B. So as an initial guess, choose sigma =0.
//   Then calculate, the point (g) that is lowest below the curve.
// Recalculate B so that the curve lies above this.
//   The original value of B we will call Bmin.
//   Then define   V' = V + Bmin-B
//   Now choose   sigmaMax=   -T/sMax
//                  1.   Initial Values
//   Step through values of sigma  from 0 to sigmaMax by steps of 1 %
//           define B=  V'-Ts -sigma s^2
//                 n1 =  -T -2*sigma s
//              find sigma that Minimizes the residual
//
//               2.  Refinement 
//     Change B n1 and sigma by 10% at each trip through
//         Minimize the Residual

// 
// 0
	sigma = 0. ;
	float   sMax = sData[NumPointsCut-1];
	float datasMax= log(IntData[NumPointsCut-1]);
	n1 =  -tailLogSlope - 2*sigma *sMax;
	if (n1<0) n1 = 1.;
	B =  datasMax - tailLogSlope*sMax - sigma*sMax*sMax;
//			
//			
	
	printf("Fitn: Ismin=%d, sData[Ismin]=%f, sigma=%f, n1=%f, B=%f, sMax=%f, datasMax=%f \n",
		Ismin,sData[Ismin],sigma,n1,B,sMax, datasMax); 

// 0b
//   Then calculate, the point (sData[iMin],IntData[Imin]) that is lowest below the curve.
// Recalculate B so that the curve lies above this.
//   This value of B we will call Bmin.
//   Then define   V' = V + Bmin-B
//   Now choose   sigmaMax=   -T/sMax
	for (int i=Ismin; i < NumPointsCut; i++){
		ResTemp  = IntData[i] - NCurve(sData[i],B,n1,sigma);     // Want data to have positive scattering
		if (ResTemp < ResMin || i == Ismin) { ResMin = ResTemp; iMin= i;} 
	}


	Bmin = B + log(IntData[iMin] / NCurve(sData[iMin],B,n1,sigma)   );

	datasMax +=  Bmin - B ;
	sigmaMax = - tailLogSlope/sMax/20.;
	if (sigmaMax <0) sigmaMax = -sigmaMax/10.;
	
	printf("Fitn: ResMin=%f, datasMax=%f, iMin=%d, sData[iMin]=%f, sigmaMax=%f, Bmin=%f \n",
	ResMin, datasMax, iMin, sData[iMin], sigmaMax, Bmin);
	sigmaBest=sigma; n1Best=n1; BBest=Bmin;	
		
//           1. 
//   Step through values of sigma  from 0 to sigmaMax by steps of 1 %
//           define B=  V'-Ts -sigma s^2
//                 n1 =  -T -2*sigma s
//              find sigma that Minimizes the residual
	ResMin= -1;
	for (int iSigma=2; iSigma < 100; iSigma ++){
		sigma = sigmaMax*( ( (float) iSigma)/100.);
			B =  datasMax - tailLogSlope*sMax - sigma*sMax*sMax;
		   n1 =  -tailLogSlope - 2.*sigma *sMax;
			Res2 = Residual(sData,IntData,B,n1,sigma,NumPoints,Ismin);
			if (DEBUG*0) printf("Fitn: iSigma=%d,  sigma=%f, Res2=%f \n",iSigma,sigma,Res2);
		if (Res2>0  && (Res2 < ResMin || ResMin== -1) ) {
					ResMin = Res2;
					sigmaBest=sigma; n1Best=n1; BBest = B;
//					printf("Fitn: sigmaBest=%f, n1Best=%f,  BBest=%f   \n",sigmaBest,n1Best,BBest); 
			}
	}
	sigma=sigmaBest; n1=n1Best; B=BBest; IFirstZero = 14;	

	*B9 = BBest; *n19= n1Best; *sigma9= sigmaBest; *IFirstZero9 = IFirstZero;

	
	if (DEBUG) printf("Fitn: Ismin=%d, sigma=%f, n1=%f, B=%f  After first guess for sigma \n",
		Ismin,sigma,n1,B); 


//               2.  Refinement 
//     Change B n1 and sigma by 10% at each trip through
//         Minimize the Residual
	
	float * quantifier = (float *) calloc(NumPoints,sizeof(float)) ;
	IFirstZero = Ismin+3;iPass=0;

	
//    Determine Ismin		
//   %%  Now count backward from the end. For each point,  find
//     the ratio of the difference of the distance to the curve to the next largest peak
//     Keep track of the point that has the smallest s but still has a ratio of difference to next
//        peak that is minimized %%

		if (DEBUG) printf("Fitn: iPass=%d, Ismin=%d, sigma=%f, n1=%f, B=%f, ResMin=%f,IFirstZero=%d \n",
			iPass,Ismin,sigma,n1,B,ResMin,IFirstZero); 
		int p4;

		for (p4 = NumPointsCut-1; p4>=Ismin; p4--) {
			residual = IntData[p4] - NCurve(sData[p4],B,n1,sigma);
			MaxPeakVal =0;
			for (int PeakLoop= p4-1; PeakLoop>=0;  PeakLoop--){
				if ( IntData[PeakLoop] - NCurve(sData[PeakLoop],B,n1,sigma) > MaxPeakVal) {
					MaxPeakVal = IntData[PeakLoop]  - NCurve(sData[PeakLoop],B,n1,sigma) ;
				} // MaxPeakValue identified
				if (MaxPeakVal >.01) quantifier[p4] = residual/MaxPeakVal; else 
					quantifier[p4] = 1;
			}
		}
		
		int p4Min= Ismin;
		for (p4 = NumPointsCut-5; p4 >= p4Min; p4--) {
//				if (DEBUG) printf("FitN:p4=%d, sData=%f, citerion=%f \n",
//					p4, sData[p4],2*quantifier[p4]- (quantifier[p4+1]+quantifier[p4-1]) );
			if (quantifier[p4]*2  < quantifier[p4+1]+quantifier[p4+2]  && quantifier[p4] <.05) {
		// Have local minimum and less than .1 and closest to the origin
				if (DEBUG) printf("FitN:p4=%d, sData=%f, quantifier=%f \n",p4, sData[p4],quantifier[p4]);
				Ismin= p4 -3;     // CHANGE ME (2 lines)
				IFirstZero = p4; // %% This is the improved value of starting the comparison 

		}}
//       Finish Determining Ismin
	
		printf("Fitn: Finished Determining Ismin=%d \n",Ismin); 

		for (int iPrintQuant = 0 ; iPrintQuant< NumPointsCut ; iPrintQuant++) {
			if (DEBUG*0) printf("FitN: iPrintQuant=%d, sData=%f, quantifier=%f \n",
				iPrintQuant, sData[iPrintQuant],quantifier[iPrintQuant]);
		}
		
		ResMin= -1;
	for (iPass=0; iPass<20; iPass++) {

		for (int jB =-numtrials; jB < numtrials; jB ++){
			B = BBest * (1. + ((float) jB)/(10.*numtrials)  );

			for (int jn1 =-numtrials; jn1 < numtrials; jn1 ++){ 
				n1= n1Best * (1. + ((float) jn1)/(10.*numtrials)  );

				for (int jsigma =-numtrials; jsigma < numtrials; jsigma ++){
					sigma = sigmaBest * (1. + ((float) jsigma)/(10.*numtrials) );

					Res2 = Residual(sData,IntData,B,n1,sigma,NumPoints,IFirstZero);
					if (Res2>0  && (Res2 < ResMin || ResMin==-1) ) {
						ResMin = Res2;
						n1BestNew=n1;
						sigmaBestNew=sigma;
			 			BBestNew = B;
						printf("Fitn: Improving %d \r",iPass); fflush(stdout);  //   
					}
	
//				printf("Fitn: iPass=%d, Ismin=%d, sigma=%f, n1=%f, B=%f, ResMin=%f,IFirstZero=%d, jn1=%d\n",
//						iPass,Ismin,sigma,n1,B,ResMin,IFirstZero,jn1); 
				}
			}
		}
		if (n1Best==n1BestNew && BBest==BBestNew && sigmaBest== sigmaBestNew) break;
		n1Best=n1BestNew;
		BBest= BBestNew; 	
		sigmaBest=sigmaBestNew;
				
		
	
	
		if (DEBUG) printf("Fitn: iPass=%d, Ismin=%d, sData[Ismin]=%f, sigma=%f, n1=%f, B=%f, ResMin=%f,IFirstZero=%d \n",
			iPass,Ismin, sData[Ismin],sigmaBest,n1Best,BBest,ResMin,IFirstZero); 

	}  //ends the iPass loop			

		if (DEBUG) printf("Fitn: iPass=%d, Ismin=%d, sData[Ismin]=%f, sigma=%f, n1=%f, B=%f, ResMin=%f,IFirstZero=%d, \
			sData[IfirstZero]=%f \n",iPass,Ismin, sData[Ismin],sigma,n1,B,ResMin,IFirstZero,sData[IFirstZero]); 



	*B9 = BBest; *n19= n1Best; *sigma9= sigmaBest; *IFirstZero9 = IFirstZero;

}


/*
	
void FitNoise2(int NumPoints, int Ismin, float * sData, float *IntData, float *B9, float *n19, float *sigma9, int *IFirstZero9) {

//      %%  1.  Now we fit the noise part:         the amplitude, exp decay, and Gaussian decay  
//       %% Make Two passes

	float s1,s2, s3, g1, g2,g3, denom,  Res2, ResMin=-1;
	int p1Best, IFirstZero; 
	float sigmaBest, n1Best, BBest;
	float B, sigma, n1;
	float taillogSlope = (log ((IntData[NumPoints-1]) /(IntData[NumPoints-4] )) )/( sData[NumPoints-1] - sData[NumPoints-4]);
	float residual, MaxPeakVal;

	for (int iPass=0; iPass <2; iPass++) {
		for (int p1=Ismin; p1 < NumPoints; p1++ ) { // printf("p1=%d \n",p1);
			s1 = sData[p1]  ;
//			printf(" s1 = %f, g1 = %f \n",sData[p1],IntData[p1]);
			if (p1==NumPoints-1){ printf("at end of p1, Ismin= %d, NumPoints=%d \n",Ismin,NumPoints); fflush(stdout);}
			for (int p2=p1+1; p2 < NumPoints; p2++ ) {
 				int p3min= p2+1; if (NumPoints*3> 4*p3min) p3min = (3*NumPoints)/4;
				for (int p3=p3min; p3 < NumPoints; p3++ ) {
			
					s2 = sData[p2]  ; s3 = sData[p3] ;
//
//  	 g = ln f = B - n1 s - sigma s^2   //
//			
					g1 = log(IntData[p1]) ; g2 = log(IntData[p2]) ; g3 = log(IntData[p3] );
					denom = (s2-s1)*(s3-s1)*(s3-s2);
			
					sigma  = (g1*(s2-s3) + g2*(s3-s1)+ g3*(s1-s2))/denom;
					n1 = (g1*(s3*s3-s2*s2) + g2*(s1*s1-s3*s3)+ g3*(s2*s2-s1*s1))/denom;
					B      = (g1*s2*s3*(s3-s2) +g2*s3*s1*(s1-s3) + g3*s1*s2*(s2-s1))/denom;
				
//     Now calculate the residual using these parameters
				
					Res2 = Residual(sData,IntData,B,n1,sigma,NumPoints,Ismin);
				if (Res2>0  ) {   	
//					printf("g1=%4.4e	Res2=%4.4e  sigma=%4.4e B=%4.4e n1=%4.4e \n", g1,  Res2, sigma, B, n1);
				}


					if (Res2>0  && (Res2 < ResMin || ResMin==-1)  &&(pabs(n1 +2 *sData[NumPoints-2] 
						* B + taillogSlope) <pabs(n1))) {  
				 // first time to set ResMin or Res2 suff small
						ResMin = Res2;
						p1Best= p1;
						sigmaBest=sigma; n1Best=n1; BBest = B;
//						printf("improved!!!  p1Best=%d, p2Best=%d, p3Best=%d, ResMin=%f \n",p1Best,p2Best,p3Best,ResMin);
//					printf("%d	%4.4e	%4.4e 	%4.4e \n",p1, B, sigma, n1);
					}
				
		}}}
		Ismin = p1Best-3; // %% This is the improved value of starting the comparison 
		B = BBest; n1= n1Best; sigma=sigmaBest;		
	
	
//   %%  Now count backward from the end. For each point,  find
//     the ratio of the difference of the distance to the curve to the next largest peak
//     Keep track of the point that has the smallest s but still has a ratio of difference to next
//        peak that is minimized %%

	float * quantifier = (float *) calloc(NumPoints,sizeof(float)) ;
	
	for (int p4 = NumPoints-1; p4>=0; p4--) {
		residual = IntData[p4] - NCurve(sData[p4],B,n1,sigma);
		MaxPeakVal =0;
		for (int PeakLoop= p4-1; PeakLoop>=0;  PeakLoop--){
			if ( IntData[PeakLoop] - NCurve(sData[PeakLoop],B,n1,sigma) > MaxPeakVal) {
				MaxPeakVal = IntData[PeakLoop]  - NCurve(sData[PeakLoop],B,n1,sigma) ;
			} // MaxPeakValue identified
			if (MaxPeakVal >.01) quantifier[p4] = residual/MaxPeakVal; else 
				quantifier[p4] = 1;
		}
	}
		

	for (p4 = NumPoints-5; p4 >= 10; p4--) {
		if (quantifier[p4]*2  < quantifier[p4+1]+quantifier[p4+2]  && quantifier[p4] <.05) {
		// Have local minimum and less than .1 and closest to the origin
			Ismin= p4 -3;
			IFirstZero = p4; // %% This is the improved value of starting the comparison 
if (DEBUG)			printf("Ismin= %d  , IFirstZero=%d sData[p4]=%f, quantifier[p4]=%f \n",
				Ismin, IFirstZero, sData[p4], quantifier[p4]);
		}
	}
		


	p1Best=Ismin;



	} // end iPass loop



	*B9 = B; *n19= n1; *sigma9= sigma; *IFirstZero9 = IFirstZero;

}
*/
