/******************************************************************************
*******************************************************************************
*                   mrc.h:  header for mrc.c                                  *
*******************************************************************************
*                            Steve Hardt                                      *
*                            East Campus                                      *
*                            3 Ames St                                        *
*                            Cambridge, MA 02139                              *
*                            (617)225-6368                                    *
*                            hardts@athena.mit.edu                            *
*                                                                             *
*                            August 24, 1992                                  *
*******************************************************************************
* Some code from mrc.h and mrc.c was taken from xvmrc.h and xvmrc.c which     *
* were written by                                                             *
*                            Man Wei Tam                                      *
*                            72 Trafalgar Rd, Wallasey,                       *
*                            Merseyside, England,                             *
*                            UK, L44OEB                                       *
*******************************************************************************
******************************************************************************/


#ifndef MRCP_H
#define MRCP_H

/* for MRC header */
#define MRC_LABEL_SIZE      80   
#define MRC_USER            29
#define MRC_NUM_LABELS      10

/*  The different modes supported by the MRC format. */
#define MODE_char           0
#define MODE_short          1
#define MODE_float          2
#define MODE_short_COMPLEX  3  
#define MODE_float_COMPLEX  4
/* All modes before MODE_short_COMPLEX must be real, all those after it must 
be complex. */


/*******************************************************************************
Protected Data Structures
*******************************************************************************/
struct mrcH
{
    int         nx;                 /* Number of columns -      */
                                    /* Fastest changing in map. */
    int         ny;

    int         nz;                 /* Number of sections -     */
                                    /* slowest changing in map. */

    int         mode;               /* See modes above. */

    int         nxstart;            /* No. of first column in map, default 0.*/
    int         nystart;            /* No. of first row in map, default 0.*/
    int         nzstart;            /* No. of first section in map,default 0.*/

    int         mx;                 /* Number of intervals along X. */
    int         my;                 /* Number of intervals along Y. */
    int         mz;                 /* Number of intervals along Z. */

    float       xlen;               /* Cell dimensions (Angstroms). */
    float       ylen;               /* Cell dimensions (Angstroms). */
    float       zlen;               /* Cell dimensions (Angstroms). */

    float       alpha;              /* Cell angles (Degrees). */
    float       beta;               /* Cell angles (Degrees). */
    float       gamma;              /* Cell angles (Degrees). */

    int         mapc;               /* Which axis corresponds to Columns.  */
                                    /* (1,2,3 for X,Y,Z.                   */
    int         mapr;               /* Which axis corresponds to Rows.     */
                                    /* (1,2,3 for X,Y,Z.                   */
    int         maps;               /* Which axis corresponds to Sections. */
                                    /* (1,2,3 for X,Y,Z.                   */

    float       amin;               /* Minimum density value. */
    float       amax;               /* Maximum density value. */
    float       amean;              /* Mean density value.    */

    int         ispg;               /* Space group number (0 for images). */

    int         nsymbt;             /* Number of chars used for storing   */
                                    /* symmetry operators. */


    int  user[MRC_USER];  

    float       xorigin;            /* X origin. */
    float       yorigin;            /* Y origin. */

    int         nlabl;              /* Number of labels being used. */
                                    /* 10 text labels of 80 characters each. */
    char        labels[MRC_NUM_LABELS][MRC_LABEL_SIZE];
    
};
#endif
