#include "PyCylinder.h"
#include "PyEMData.h"
#include <boost/python/detail/extension_class.hpp>
#include "PyPickle.h"

using std::vector;

BOOST_PYTHON_BEGIN_CONVERSION_NAMESPACE
static python_extension_class_converters<PyCylinder> _PyCylinder_type;
static python_extension_class_converters<PyCylinders> _PyCylinders_type;
static python_extension_class_converters<PyObjectStat> _PyObjStat_type;
static python_extension_class_converters<PyVect> _PyVect_type;
BOOST_PYTHON_END_CONVERSION_NAMESPACE

PyCylinder::PyCylinder() : Cylinder() { }

PyCylinder::PyCylinder(const PyVect& center, const PyVect& axisdir,
		       float len, float r, float w)
    : Cylinder(center, axisdir, len, r, w) { }

PyVect PyCylinder::py_Center()
{
    Vect v = Center();
    return *(static_cast<PyVect*>(&v));
}

PyVect PyCylinder::py_Axis()
{
    Vect v = Axis();
    return *(static_cast<PyVect*>(&v));
}

PyQuater PyCylinder::py_QuaterForAxis()
{
    Quater q = QuaterForAxis();
    return *(static_cast<PyQuater*>(&q));    
}

float PyCylinder::py_PlaneAngleDiff(PyCylinder& next)
{
    return PlaneAngleDiff(&next);    
}

float PyCylinder::py_AxisAngleDiff(PyCylinder& next)
{
    return AxisAngleDiff(&next);    
}

float PyCylinder::py_Angle2Center(PyCylinder& next)
{
    return Angle2Center(&next);    
}

float PyCylinder::py_NormDistance(PyCylinder& next) 
{
    return NormDistance(&next);    
}

float PyCylinder::py_CenterDistance(PyCylinder& next) 
{
    return CenterDistance(&next);    
}

bool PyCylinder::py_if_merge(PyCylinder& next, float da_limit,
			     float distance_limit) 
{
    return if_merge(&next, da_limit, distance_limit);     
}

bool PyCylinder::py_merge(PyCylinder& next)
{
    return merge(&next);    
}

void PyCylinder::py_setCenter(PyVect& cen)
{
    setCenter(cen.X(), cen.Y(), cen.Z());
}
    

const char* PyCylinder::py_doc(const char* funcname)
{
    if (!funcname) {
	return "";
    }

    static bool has_built = false;
    static python::dictionary dict = python::dictionary();
    
    if (!has_built) {
	dict["Cylinder"] = "Cylinder()\n construct an empty Cylinder instance\n\nCylinder(Vect center, Vect axisdir, len = 5.4, r = 2.5, w = 0.9)";
	dict["Center"] = "Vect Center()";
	dict["Axis"] = "Vect Axis()";
	dict["QuaterForAxis"] = "Quater QuaterForAxis()";
	
	dict["Xc"] = "float Xc()";
	dict["Yc"] = "float Yc()";
	dict["Zc"] = "float Zc()";
	dict["Alt"] = "float Alt()";
	dict["Az"] = "float Az()";
	dict["Radius"] = "float Radius()";
	dict["Length"] = "float Length()";
	dict["Weight"] = "float Weight()";

	dict["StartID"] = "int StartID()";
	dict["EndID"] = "int EndID()";
	
	dict["Label"] = "string	Label()";
	dict["Print"] = "Print(id = -1, out = stdout)";
	
	dict["toInventor"] = "toInventor(out = stdout, id = 0, indent = 1, with_head = false)";
	dict["toDejaVu"] = "toDejaVu(out = stdout, id = 0)";
	dict["toCosec"] = "toCosec(out = stdout, id = 0)";
	dict["setCenter"] = "setCenter(Vect center)\nsetCenter(x, y, z);";
	dict["setAxis"] = "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)";
	dict["setRadius"] = "setRadius(r)";
	dict["setLength"] = "setLength(l)";
	dict["setWeight"] = "setWeight(w)";
	dict["setStartID"] = "setStartID(i)";
	dict["setEndID"] = "setEndID(i)";
	dict["setLabel"] = "setLabel(string l)";
	
	dict["PlaneAngleDiff"] = "float PlaneAngleDiff(Cylinder next)\n  dihedral angle of planes formed by the two axes and center connection vector";

	dict["AxisAngleDiff"] = "float AxisAngleDiff(Cylinder next)\n  difference in orientation (axises)";

	dict["Angle2Vect"] = "float Angle2Vect(Vect v)\n  difference in orientation axis of this and the direction vector ";

	dict["Angle2Point"] = "float Angle2Point(Vect p)\n  difference in orientation between line passing both P and this center and axis of this";
	
	dict["Angle2Center"] = "float Angle2Center(Cylinder next)\n  difference in orientation between line passing both centers and axis of this";

	dict["NormDistance"] = "float NormDistance(Cylinder next)\n  shortest distance of the two axes ";
	dict["CenterDistance"] = "float	CenterDistance(Cylinder next)\n  distance of the center of next cylinder to the center of this";

	dict["Distance2Point"] = "float	Distance2Point(x, y, z)\n  distance of the Point to the center of this cylinder";

	dict["if_merge"] = "bool if_merge(Cylinder next, da_limit = 30*PI/180., distance_limit = 3.2)\n  if next is ok to merge with this";

	dict["merge"] = "bool merge(Cylinder next)\n  actually merge both into this";

	dict["mirror"] = "mirror()\n  mirror the cylinders";
	
	dict["Transform"] = "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate";
	
	
	has_built = true;
    }
    
    const char* fdoc = BOOST_PYTHON_CONVERSION::from_python(dict.get_item(funcname).get(),
							    python::type<const char*>());
    if (!fdoc) {
	fdoc = "Error: not defined";
    }
    return fdoc;
}


// class PyCylinders

PyCylinders PyCylinders::py_copy()
{
    Cylinders c = copy();
    return *(static_cast<PyCylinders*>(&c));
}

PyCylinders PyCylinders::py_subset(PyList data_list, int n)
{
#ifdef DEBUG
    assert(data_list.size() >= n);
#endif
    int* data_array = new int[data_list.size()];
    list2array<int>(data_list, data_array);
    Cylinders c = subset(data_array, n);
    array2list<int>(data_array, data_list);
    
    delete [] data_array;
    data_array = 0;
    
    return *(static_cast<PyCylinders*>(&c));
}

bool PyCylinders::py_add(PyCylinder& cyl)    
{
    return add(&cyl);
}

bool PyCylinders::py_ReadListfile(const char * in)
{
    //return ReadListfile((char*) in);
    return ReadListfile(const_cast<char*>(in));
}

bool PyCylinders::py_WriteListfile(const char * out)
{
    return WriteListfile(const_cast<char*>(out));
}

bool PyCylinders::py_ReadInventor(const char * in)
{
    return ReadInventor(const_cast<char*>(in));
}

bool PyCylinders::py_WriteInventor(const char * out)
{
    return WriteInventor(const_cast<char*>(out));
}

bool PyCylinders::py_WriteVRML(const char *out)
{
    return WriteVRML(const_cast<char*>(out));
}

bool PyCylinders::py_ReadDejaVu(const char * in)
{
    return ReadDejaVu(const_cast<char*>(in));
}

bool PyCylinders::py_ReadCosec(const char * in)
{
    return ReadCosec(const_cast<char*>(in));
}


PyCylinder_ptr PyCylinders::get(int i)
{
    Cylinder* c = (*this)[i];
    PyCylinder* py_c = static_cast<PyCylinder*>(c);
    return PyCylinder_ptr(py_c);
}


const char* PyCylinders::py_doc(const char* funcname)
{
    if (!funcname) {
	return "";
    }

    static bool has_built = false;
    static python::dictionary dict = python::dictionary();
    
    if (!has_built) {
	dict["size"] = "size()";
	dict["copy"] = "Cylinders copy()";
	dict["subset"] = "Cylinders subset(list1, int n)";
	dict["add"] = "bool add(Cylinder cyl)";
	dict["ReadListfile"] = "bool ReadListfile(in = '-')\nbool ReadListfile(in = stdin)";
	dict["WriteListfile"] = "bool WriteListfile(out = stdout)\nbool WriteListfile(out = '-')";
	dict["ReadInventor"] = "bool ReadInventor(in = stdin)\nbool ReadInventor(in = '-')";
	dict["WriteInventor"] = "bool WriteInventor(out = stdout)\nbool WriteInventor(out = '-')";
	dict["WriteVRML"] = "bool WriteVRML(out = stdout)\nbool WriteVRML(out = '-')";
	dict["ReadDejaVu"] = "bool ReadDejaVu(in=stdin)\nbool ReadDejaVu(in='-')";
	;
	dict["WriteDejaVu"] = "bool WriteDejaVu(out=stdout, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(out='-', mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')";
	dict["ReadCosec"] = "bool ReadCosec(in=stdin)\nbool ReadCosec(in='-')";
	;
	dict["WriteCosec"] = "bool WriteCosec(out=stdout, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(out='-', prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')";
	;
	dict["SortLength"] = "bool SortLength()";
	dict["SortWeight"] = "bool SortWeight()";
	;
	dict["OrientationDiff"] = "float OrientationDiff(Cylinders comp, ignoredir=true, uselengthweight=true)\n  average individual cylinders angle diff";
	dict["mirror"] = "mirror()";
	dict["Transform"] = "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate";
	;
	dict["Print"] = "Print()";

	
	has_built = true;
    }
    
    const char* fdoc = BOOST_PYTHON_CONVERSION::from_python(dict.get_item(funcname).get(),
							    python::type<const char*>());
    if (!fdoc) {
	fdoc = "Error: not defined";
    }
    return fdoc;
}


// class PyObjectStat


void PyObjectStat::py_SetObject(PyEMData& map, const PyList& obj_list)
{
    vector<Point*> vect;
    for (int i = 0; i < obj_list.size(); i++) {
	PyObject* obj = obj_list.get_item(i).get();
	PyVect* item = BOOST_PYTHON_CONVERSION::from_python(obj, boost::python::type<PyVect*>());
	vect.push_back(item);
    }
    SetObject(&map, vect);
}

void PyObjectStat::py_SetObject(PyEMData& map)
{
    SetObject(&map);
}


void PyObjectStat::py_EigenSortTo(PyObjectStat& obj)
{
    EigenSortTo(&obj);
}


const char* PyObjectStat::py_doc(const char* funcname)
{
    if (!funcname) {
	return "";
    }

    static bool has_built = false;
    static python::dictionary dict = python::dictionary();

    
    if (!has_built) {
	dict["SetObject"] = "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)";
	dict["EigenSortTo"] = "EigenSortTo(ObjectStat obj)";
	dict["FitEllipsoid"] = "FitEllipsoid(sort = true)";
	dict["Norm"] = "Norm()";
	dict["ComputeLongAxis"] = "ComputeLongAxis()";
	dict["Print"] = "Print()";
	
	has_built = true;
    }
    
    const char* fdoc = BOOST_PYTHON_CONVERSION::from_python(dict.get_item(funcname).get(),
							    python::type<const char*>());
    if (!fdoc) {
	fdoc = "Error: not defined";
    }
    return fdoc;
}




enum CylinderState {
    CYLINDER_CEN,
    CYLINDER_AXIS,
    CYLINDER_RADIUS,
    CYLINDER_LENGTH,
    CYLINDER_WEIGHT,
    CYLINDER_STARTID,
    CYLINDER_ENDID,
    CYLINDER_LABEL,
    CYLINDER_DICT,
    CYLINDER_NUM_STATES
};



python::ref Cylinder_getstate(const python::tuple& args,
			      const python::dictionary& keywords)
{
    check_args(args, keywords, 1);

    PyCylinder& cyl = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							   python::type<PyCylinder&>());
    python::tuple result(CYLINDER_NUM_STATES);
    
    Vect cen = cyl.Center();
    PyVect pcen = *((PyVect*)&cen);
   
    result.set_item(CYLINDER_CEN, pcen);

    Vect axis = cyl.Axis();
    PyVect paxis = *((PyVect*)&axis);;    
    result.set_item(CYLINDER_AXIS, paxis);
    
    result.set_item(CYLINDER_RADIUS, cyl.Radius());
    result.set_item(CYLINDER_LENGTH, cyl.Length());
    result.set_item(CYLINDER_WEIGHT, cyl.Weight());
    result.set_item(CYLINDER_STARTID, cyl.StartID());
    result.set_item(CYLINDER_ENDID, cyl.EndID());
    result.set_item(CYLINDER_LABEL, cyl.Label());
    
    python::ref mydict = python::getattr(args[0], "__dict__");
    result.set_item(CYLINDER_DICT, mydict);
    
    return result.reference();
}

PyObject* Cylinder_setstate(const python::tuple& args,
			    const python::dictionary& keywords)
{
    set_dict(args, keywords, CYLINDER_NUM_STATES);

    PyCylinder& cyl = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							   python::type<PyCylinder&>());  
    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(),
							       python::type<python::tuple>());
    
    PyVect pcen = BOOST_PYTHON_CONVERSION::from_python(state[CYLINDER_CEN].get(),
						       python::type<PyVect>());
    cyl.py_setCenter(pcen);

    PyVect paxis = BOOST_PYTHON_CONVERSION::from_python(state[CYLINDER_AXIS].get(),
							python::type<PyVect>());
    Vect axis = *((Vect*)&paxis);
    cyl.setAxis(axis);

    cyl.setRadius(get_float(state[CYLINDER_RADIUS]));
    cyl.setLength(get_float(state[CYLINDER_LENGTH]));
    cyl.setWeight(get_float(state[CYLINDER_WEIGHT]));
    cyl.setStartID(get_int(state[CYLINDER_STARTID]));
    cyl.setEndID(get_int(state[CYLINDER_ENDID]));

    string label = BOOST_PYTHON_CONVERSION::from_python(state[CYLINDER_LABEL].get(),
							python::type<string>());
    cyl.setLabel(label);

    return python::detail::none();
}


    
python::ref Cylinders_getstate(const python::tuple& args,
			       const python::dictionary& keywords)
{
    check_args(args, keywords, 1);
    PyCylinders& cyls = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							     python::type<PyCylinders&>());
    python::tuple result(2);
    int sz = cyls.size();
    
    PyList cyl_list(sz);
    for (int i = 0; i < sz; i++) {
	PyCylinder* pcyl = static_cast<PyCylinder*>(cyls[i]);
	cyl_list.append(*pcyl);
    }
    result.set_item(0, cyl_list);
    
    python::ref mydict = python::getattr(args[0], "__dict__");
    result.set_item(1, mydict);
    
    return result.reference();
}

PyObject* Cylinders_setstate(const python::tuple& args,
			     const python::dictionary& keywords)
{
    set_dict(args, keywords, 2);
    PyCylinders& cyls = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							     python::type<PyCylinders&>());  
    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(),
							       python::type<python::tuple>());

    PyList cyl_list = BOOST_PYTHON_CONVERSION::from_python(state[0].get(),
							   python::type<PyList>());

    for (int i = 0; i < cyl_list.size(); i++) {
	PyCylinder pcyl = BOOST_PYTHON_CONVERSION::from_python(cyl_list.get_item(i).get(),
							       python::type<PyCylinder>());
	
	cyls.add(&pcyl);
    }
    
    return python::detail::none();
}


enum ObjectStat_State {
    OBJSTAT_XC,
    OBJSTAT_YC,
    OBJSTAT_ZC,
    OBJSTAT_AXIS,
    OBJSTAT_MASS,
    OBJSTAT_VOLUME,
    OBJSTAT_LEN,
    OBJSTAT_RAD,
    OBJSTAT_ASPECTRATIO,
    OBJSTAT_MOMENTS,
    OBJSTAT_EIGENVALUE,
    OBJSTAT_EIGENVECTOR,
    OBJSTAT_DICT,
    OBJSTAT_NUM_STATES
};
    
python::ref ObjectStat_getstate(const python::tuple& args,
				const python::dictionary& keywords)
{
    check_args(args, keywords, 1);
    PyObjectStat& os = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							    python::type<PyObjectStat&>());
    python::tuple result(OBJSTAT_NUM_STATES);

    result.set_item(OBJSTAT_XC, os.xc);
    result.set_item(OBJSTAT_YC, os.yc);
    result.set_item(OBJSTAT_ZC, os.zc);

    PyVect paxis = *((PyVect*) &os.axis);
    result.set_item(OBJSTAT_AXIS, paxis);
    
    result.set_item(OBJSTAT_MASS, os.mass);
    result.set_item(OBJSTAT_VOLUME, os.volume);
    result.set_item(OBJSTAT_LEN, os.len);
    result.set_item(OBJSTAT_RAD, os.rad);

    PyList asp_list(3);
    array2list<float>(os.aspectratio, asp_list);    
    result.set_item(OBJSTAT_ASPECTRATIO, asp_list);

    PyList mom_list(9);
    array2list<float>((float*)os.moments, mom_list);    
    result.set_item(OBJSTAT_MOMENTS, mom_list);

    PyList evalue_list(3);
    array2list<float>(os.eigenvalue, evalue_list);    
    result.set_item(OBJSTAT_EIGENVALUE, evalue_list);

    PyList evector_list(9);
    array2list<float>((float*)os.eigenvector, evector_list);
    result.set_item(OBJSTAT_EIGENVECTOR, evector_list);
    
    python::ref mydict = python::getattr(args[0], "__dict__");    
    result.set_item(OBJSTAT_DICT, mydict);
    
    return result.reference();
}

PyObject* ObjectStat_setstate(const python::tuple& args,
			      const python::dictionary& keywords)
{
    set_dict(args, keywords, OBJSTAT_NUM_STATES);
    PyObjectStat& os =  BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							     python::type<PyObjectStat&>());    
    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(),
							       python::type<python::tuple>());
    
    os.xc = get_float(state[OBJSTAT_XC]);
    os.yc = get_float(state[OBJSTAT_YC]);
    os.zc = get_float(state[OBJSTAT_ZC]);

    PyVect paxis = BOOST_PYTHON_CONVERSION::from_python(state[OBJSTAT_AXIS].get(),
							python::type<PyVect>());
    os.axis = *((Vect*)&paxis);

    os.mass = get_float(state[OBJSTAT_MASS]);
    os.volume = get_float(state[OBJSTAT_VOLUME]);
    os.len = get_float(state[OBJSTAT_LEN]);
    os.rad = get_float(state[OBJSTAT_RAD]);

    PyList asp_list = BOOST_PYTHON_CONVERSION::from_python(state[OBJSTAT_ASPECTRATIO].get(),
							   python::type<PyList>());
    list2array<float>(asp_list, os.aspectratio);
    
    PyList mom_list = BOOST_PYTHON_CONVERSION::from_python(state[OBJSTAT_MOMENTS].get(),
							   python::type<PyList>());
    list2array<float>(mom_list, (float*)os.moments);
    
    PyList evalue_list = BOOST_PYTHON_CONVERSION::from_python(state[OBJSTAT_EIGENVALUE].get(),
							      python::type<PyList>());
    list2array<float>(evalue_list, os.eigenvalue);
    
    PyList evector_list = BOOST_PYTHON_CONVERSION::from_python(state[OBJSTAT_EIGENVECTOR].get(),
							       python::type<PyList>());
    list2array<float>(evector_list, (float*)os.eigenvector);
    
    return python::detail::none();
}



