#include "PyPickle.h"

namespace boost { namespace python {

    ref getattr(PyObject* o, const std::string& attr_name) {
	return ref(PyObject_GetAttrString(o, const_cast<char*>(attr_name.c_str())));
    }
    ref getattr(const ref& r, const std::string& attr_name) {
	return getattr(r.get(), attr_name);
    }

}}

void check_args(const python::tuple& args,
		const python::dictionary& keywords, int narg)
{
    if (args.size() != narg || keywords.size() != 0) {
	PyErr_SetString(PyExc_TypeError, "wrong number of arguments");
	throw python::error_already_set();
    }
}


void set_dict(const python::tuple& args, const python::dictionary&
	      keywords, int nstates)
	      
{
    check_args(args, keywords, 2);

    python::ref mydict = boost::python::getattr(args[0], "__dict__");
    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(), python::type<python::tuple>());
    
    if (state.size() != nstates) {
	PyErr_SetString(PyExc_ValueError,
			"Unexpected argument in call to __setstate__.");
        throw python::error_already_set();
    }
    
    python::dictionary odict = BOOST_PYTHON_CONVERSION::from_python(mydict.get(), python::type<python::dictionary>());
    const python::dictionary& pdict =
	BOOST_PYTHON_CONVERSION::from_python(state[nstates-1].get(), python::type<const python::dictionary&>());

    python::list pkeys(pdict.keys());
    for (int i = 0; i < pkeys.size(); i++) {
	python::ref k(pkeys[i]);
	odict[k] = pdict.get_item(k);
    }
}


float get_float(python::ref r)
{
    return BOOST_PYTHON_CONVERSION::from_python(r.get(), python::type<float>());
}

int get_int(python::ref r)
{
    return BOOST_PYTHON_CONVERSION::from_python(r.get(), python::type<int>());
}

const char* get_string(python::ref r)
{
    return BOOST_PYTHON_CONVERSION::from_python(r.get(), python::type<const char*>());
}

long get_long(python::ref r)
{
    return BOOST_PYTHON_CONVERSION::from_python(r.get(), python::type<long>());
}
