#include "PyVect.h"
#include "PyPickle.h"

PyQuater::PyQuater() : Quater() {}

PyQuater::PyQuater(PyEuler& e) : Quater(e) { }

PyQuater::PyQuater(float x, float y, float z, float q) :
    Quater(x, y, z, q) { }

void PyQuater::py_fromEuler(PyEuler& e)
{
    fromEuler(e);
}

PyEuler PyQuater::py_toEuler()
{
    Euler e = toEuler();
    PyEuler* pye = static_cast<PyEuler*>(&e);
    return *pye;
}

void PyQuater::py_print(const char* id, FILE* out)
{
    print(const_cast<char*>(id), out);
}


float PyQuater::get_nx() const 
{
    return nx;
}

float PyQuater::get_ny() const
{
    return ny;
}
float PyQuater::get_nz() const
{
    return nz;
}
float PyQuater::get_Q() const
{
    return Q;
}

void PyQuater::set_nx(float x)
{
    nx = x;
}

void PyQuater::set_ny(float y)
{
    ny = y;
}
void PyQuater::set_nz(float z)
{
    nz = z;
}
void PyQuater::set_Q(float q)
{
    Q = q;
}

const char* PyQuater::py_doc(const char* funcname)
{
   if (!funcname) {
	return "";
    }

    static bool has_built = false;
    static python::dictionary dict = python::dictionary();
    
    if (!has_built) {
	dict["Quater"] = "Quater()\nQuater(Euler e)\nQuater(x, y, z, q)";
	dict["fromEuler"] = "fromEuler(euler)";
	dict["toEuler"] = "Euler toEuler()";
	dict["Print"] = "Print(id = "", out = stdout)";
	
	has_built = true;
    }
    
    const char* fdoc = BOOST_PYTHON_CONVERSION::from_python(dict.get_item(funcname).get(),
							    python::type<const char*>());
    if (!fdoc) {
	fdoc = "Error: not defined";
    }
    return fdoc;
}
    


// class Vect

PyVect::PyVect() : Vect()
{
}

PyVect::PyVect(float xx, float yy, float zz) : Vect(xx, yy, zz)
{
    
}

PyVect PyVect::py_rotate(PyList m_list)
{
    float *m_array = new float[PyEuler::EULER_MATRIX_SIZE];
    list2array<float>(m_list, m_array);
    Vect v = rotate(m_array);
    array2list<float>(m_array, m_list);
    delete [] m_array;
    m_array = 0;
    
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(PyEuler& ang)
{
    Vect v = rotate(ang);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(PyQuater& q)
{
    Vect v = rotate(q);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(PyVect& p, float q)
{
    Vect v = rotate(p, q);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(float alt, float az, float phi)
{
    Vect v = rotate(alt, az, phi);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(float nx, float ny, float nz, float q)
{
    Vect v = rotate(nx, ny, nz, q);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_cross(PyVect& p)
{
    Vect v = cross(p);
    return *(static_cast<PyVect*>(&v));
}

PyQuater PyVect::py_getQuater()
{
    Quater q = getQuater();
    return *(static_cast<PyQuater*>(&q));
}

float PyVect::py_inner(PyVect& p)
{
    return inner(p);
}

float PyVect::py_angle(PyVect& p, PyVect y)
{
    return angle(p, y);
}

PyEuler PyVect::py_euler()
{
    Euler e = euler();
    return *(static_cast<PyEuler*>(&e));
}

PyEuler PyVect::py_eulerTo(PyVect& p)
{
    Euler e = eulerTo(p);
    return *(static_cast<PyEuler*>(&e));
}

PyEuler PyVect::py_eulerTo(float x, float y, float z)
{
    Euler e = eulerTo(x, y, z);
    return *(static_cast<PyEuler*>(&e));
}

bool PyVect::py_isNeighbor(PyVect& p)
{
    return isNeighbor(p);
}

void PyVect::py_print(const char* id, FILE* out)
{
    print(const_cast<char*>(id), out);
}



const char* PyVect::py_doc(const char* funcname)
{
   if (!funcname) {
	return "";
    }

    static bool has_built = false;
    static python::dictionary dict = python::dictionary();
    
    if (!has_built) {
	dict["Vect"] = "Vect()\nVect(float xx, float yy, float zz)";
	dict["normalize"] = " normalize()";
	
	dict["rotate"] = "Vect rotate(list euler_matrix)\nVect rotate(Euler euler)\nVect rotate(alt, az, phi)\nVect rotate(Quater q)\nVect rotate(Vect p, q)\nVect rotate(nx, ny, nz, q);";
	
	dict["cross"] = "Vect cross(Vect p)";
	dict["getQuater"] = "Quater getQuater()";
	
	dict["X"] = "X()";
	dict["Y"] = "Y()";
	dict["Z"] = "Z()";
	dict["Alt"] = "Alt()";	
	dict["Az"] = "Az()";
	
	dict["setX"] = "setX(xx)";
	dict["setY"] = "setY(yy);";
	dict["setZ"] = "setZ(zz)";
	dict["setXYZ"] = "setXYZ(xx, yy, zz)";
	dict["inner"] = "float inner(Vect p);";
	dict["angle"] = "float angle(Vect p, Vect y=Vect())";
	dict["length"] = "float length()";
	dict["euler"] = "Euler euler();";
	
	dict["eulerTo"] = "Euler eulerTo(Vect p)\nEuler eulerTo(x, y, z)";
	dict["isNeighbor"] = "bool isNeighbor(Vect p)\n  26 neighbor connection";
	
	
	has_built = true;
    }
    
    const char* fdoc = BOOST_PYTHON_CONVERSION::from_python(dict.get_item(funcname).get(),
							    python::type<const char*>());
    if (!fdoc) {
	fdoc = "Error: not defined";
    }
    return fdoc;
}


PyVect operator * (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v * f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator / (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v / f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator + (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v + f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator - (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v - f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator + (const PyVect& lhs, const PyVect& rhs)
{
    Vect v1 = lhs;
    Vect v2 = rhs;
    Vect r = v1 + v2;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator - (const PyVect& lhs, const PyVect& rhs)
{
    Vect v1 = lhs;
    Vect v2 = rhs;
    Vect r = v1 - v2;
    return *(static_cast<PyVect*>(&r));
}

bool operator == (const PyVect& lhs, const PyVect& rhs)
{
    Vect v1 = lhs;
    Vect v2 = rhs;
    return (v1 == v2);
}



BOOST_PYTHON_BEGIN_CONVERSION_NAMESPACE
static python_extension_class_converters<PyQuater> _PyQuater_type;
static python_extension_class_converters<PyVect> _PyVect_type;
BOOST_PYTHON_END_CONVERSION_NAMESPACE




static python::ref Quater_getref(PyQuater& q, bool has_dict = false,
				 python::ref mydict = python::ref())
{
    int nitems = 4;
    if (has_dict) {
	nitems++;
    }

    python::tuple r(nitems);
    r.set_item(0, q.nx);
    r.set_item(1, q.ny);
    r.set_item(2, q.nz);
    r.set_item(3, q.Q);

    if (has_dict) {
	r.set_item(4, mydict);
    }
    
    return r.reference();    
}

python::ref Quater_getinitargs(PyQuater& q)
{
    return Quater_getref(q);
}


python::ref Quater_getstate(const python::tuple& args,
			    const python::dictionary& keywords)
{
    check_args(args, keywords, 1);
    PyQuater& quater = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							    python::type<PyQuater&>());
    python::ref mydict = python::getattr(args[0], "__dict__");

    return Quater_getref(quater, true, mydict);
}

PyObject* Quater_setstate(const python::tuple& args,
			  const python::dictionary& keywords)
{
    set_dict(args, keywords, 5);
    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(),
							       python::type<python::tuple>());
    
    PyQuater& quater = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							    python::type<PyQuater&>());
    quater.nx = get_float(state[0]);
    quater.ny = get_float(state[1]);
    quater.nz = get_float(state[2]);
    quater.Q = get_float(state[3]);

    return python::detail::none();
}

static python::ref Vect_getref(PyVect& v, bool has_dict = false,
			       python::ref mydict = python::ref())
{
    int nitems = 3;
    if (has_dict) {
	nitems++;
    }
    python::tuple r(nitems);
    r.set_item(0, v.X());
    r.set_item(1, v.Y());
    r.set_item(2, v.Z());

    if (has_dict) {
	r.set_item(3, mydict);
    }
    
    return r.reference();
}


python::ref Vect_getinitargs(PyVect& v)
{
    return Vect_getref(v);
}

python::ref Vect_getstate(const python::tuple& args,
			  const python::dictionary& keywords)
{
    check_args(args, keywords, 1);

    PyVect& v = BOOST_PYTHON_CONVERSION::from_python(args[0].get(), python::type<PyVect&>());
    python::ref mydict = python::getattr(args[0], "__dict__");

    return Vect_getref(v, true, mydict);    
}

PyObject* Vect_setstate(const python::tuple& args,
			const python::dictionary& keywords)
{
    set_dict(args, keywords, 4);

    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(), python::type<python::tuple>());
    PyVect& v = BOOST_PYTHON_CONVERSION::from_python(args[0].get(), python::type<PyVect&>());
    v.setX(get_float(state[0]));
    v.setY(get_float(state[1]));
    v.setZ(get_float(state[2]));

    return python::detail::none();
}


