#include <stdlib.h>
#include <stdio.h>
#include <qvalidator.h>
#include <math.h>
#include "FEntry.h"

FEntry::FEntry(int tp,char *lab,char *range,char *txt, char *hlp, QScrollView *pw,int x,int y) {
textw=100;

type=tp;
label=lab;
text=txt;
help=hlp;

wlabel=new QLabel(text,pw);
wlabel->setAutoResize(TRUE);
//printf("%d,'%s','%s','%s',%d\n",tp,lab,range,txt,y);

if (type!=F_NONE && type!=F_TEXT && help.length()>1) {
	whelpbut=new QPushButton("?",pw->viewport());
	//whelpbut->setAutoResize(TRUE);
	whelpbut->resize(18,18);
	whelpbut->setFocusPolicy(QWidget::NoFocus);
	connect(whelpbut,SIGNAL(clicked()),this,SLOT(dohelp()));
	whelpbut->show();
}
else whelpbut=NULL;

wentry=NULL;

switch (type) {
case F_NONE:
	pw->addChild(wlabel,10,y);
	break;
case F_TEXT:
	{	
		QFont *nf = new QFont(wlabel->font());
		nf->setBold(TRUE);
		nf->setItalic(TRUE);
		wlabel->setFont(*nf);
		pw->addChild(wlabel,10,y);
//		wlabel->setText(text);
	}
	break;
case F_STRING:
case F_FLOAT:
case F_INT:
	pw->addChild(wlabel,140,y+2);

	if (whelpbut) pw->addChild(whelpbut,120,y);

	wentry=new QLineEdit(pw->viewport(),label.data());
	if (type==F_STRING) wentry->resize(100,18);
	else if (type==F_FLOAT) {
		float min=-100.0,max=100.0;
		int dig=2;
		sscanf(range," %f , %f , %d",&min,&max,&dig);
		((QLineEdit *)wentry)->setValidator(new QDoubleValidator(min,max,dig,wentry));
		textw=8*(dig+2+(int)floor(log10(max+min)));
		wentry->resize(textw,18);
	}
	else {
		int min=-100,max=100;
		sscanf(range," %d , %d",&min,&max);
		((QLineEdit *)wentry)->setValidator(new QIntValidator(min,max,wentry));
		textw=8*((int)floor(1+log10(max+min)));
		wentry->resize(textw,18);
	}
	
	((QLineEdit *)wentry)->setEchoMode( QLineEdit::Normal );
	((QLineEdit *)wentry)->setFocusPolicy(QWidget::StrongFocus);
	connect(wentry,SIGNAL(returnPressed()),pw,SLOT(change()));
	connect(wentry,SIGNAL(textChanged(const QString &)),pw,SLOT(change(const QString &)));
//	pw->addChild(wentry,0,y+wlabel->height()/2-8);
	pw->addChild(wentry,110-textw,y);

	break;
case F_BUTTON:
	pw->addChild(wlabel,140,y+2);

	if (whelpbut) pw->addChild(whelpbut,120,y);

	wentry=new QPushButton(pw->viewport(),label.data());
	wentry->resize(40,22);
	((QPushButton *)wentry)->setAutoResize( TRUE );
	((QPushButton *)wentry)->setText( lab );
	connect(wentry,SIGNAL(clicked()),pw,SLOT(change()));
//	pw->addChild(wentry,0,y+wlabel->height()/2-8);
	pw->addChild(wentry,110-wentry->width(),y);

	break;
case F_LIST:
	int i,j;
	
	pw->addChild(wlabel,140,y+2);

	if (whelpbut) pw->addChild(whelpbut,120,y);

	wentry=new QComboBox(pw->viewport(),label.data());
//	((QComboBox*)wentry)->setAutoResize(TRUE);
	for (i=j=0; range[i]!=0; i++) {
		if (range[i]==',') {
			range[i]=0;
			((QComboBox*)wentry)->insertItem(range+j);
			if (strlen(range+j)>(textw-10)/8) textw=10+strlen(range+j)*8;
			j=i+1;
		}
	}
	if (textw>100) textw=100;
	wentry->resize(textw,22);
	((QComboBox*)wentry)->insertItem(range+j);
//	pw->addChild(wentry,0,y+wlabel->height()/2-8);
	connect(wentry,SIGNAL(activated(int)),pw,SLOT(change(int)));
	pw->addChild(wentry,110-wentry->width(),y-2);
	break;
}

}

FEntry::~FEntry() {

if (wentry) delete(wentry);
if (wlabel) delete(wlabel);
if (whelpbut) delete(whelpbut);

}

int FEntry::height() {
return (wlabel->height()>18?wlabel->height():18);
}

int FEntry::width() {
if (type==F_NONE || type==F_TEXT) return (wlabel->width()+10);
return (wlabel->width()+140);
}

int FEntry::intValue() {
switch (type) {
case F_NONE:
	return 0;
	break;
case F_STRING:
case F_FLOAT:
case F_INT:
	return(atoi(((QLineEdit *)wentry)->text()));
	break;
case F_LIST:
	return((QComboBox *)wentry)->currentItem();
	break;
}
return 0;
}

float FEntry::floatValue() {
switch (type) {
case F_NONE:
	return 0;
	break;
case F_STRING:
case F_FLOAT:
case F_INT:
	return(atof(((QLineEdit *)wentry)->text()));
	break;
case F_LIST:
	return((QComboBox *)wentry)->currentItem();
	break;
}
return 0;
}

const char *FEntry::strValue() {
switch (type) {
case F_NONE:
	return NULL;
	break;
case F_STRING:
case F_FLOAT:
case F_INT:
	return((const char *)((QLineEdit *)wentry)->text());
	break;
case F_LIST:
	return(const char *)((QComboBox *)wentry)->currentText();
	break;
}
return NULL;
}


void FEntry::changed() {
emit newValue();
}


void FEntry::dohelp() {
QMessageBox::information( NULL, "EMAN",help.data());
}

void FEntry::setIntValue(int v) {
char s[20];

switch (type) {
case F_NONE:
case F_STRING:
case F_FLOAT:
case F_INT:
	sprintf(s,"%d",v);
	((QLineEdit *)wentry)->setText(s);
	break;
case F_LIST:
	((QComboBox *)wentry)->setCurrentItem(v);
	break;
}

}

void FEntry::setFloatValue(float v) {
char s[20],t[20];
int n;

switch (type) {
case F_NONE:
case F_STRING:
case F_FLOAT:
	n=log(fabs(v)+.00001);
	if (n<=0) n=1;
	n=(textw-3)/8-n-2;
	if (n<=0) n=0;
	sprintf(t,"%%0.%0df",n);
	sprintf(s,t,v);
	((QLineEdit *)wentry)->setText(s);
	break;
case F_INT:
	sprintf(s,"%d",(int)v);
	((QLineEdit *)wentry)->setText(s);
	break;
case F_LIST:
	((QComboBox *)wentry)->setCurrentItem((int)v);
	break;
}

}

void FEntry::setStringValue(char *v) {

switch (type) {
case F_NONE:
	return ;
	break;
case F_STRING:
	((QLineEdit *)wentry)->setText(v);
	break;
case F_FLOAT:
	setFloatValue(atof(v));
	break;
case F_INT:
case F_LIST:
	setIntValue(atoi(v));
	break;
}

}

