#include <qlistbox.h>
#include "qmenubar.h"
#include "qpopupmenu.h"
#include "qspinbox.h"
#include "ControlU.h"
#include "OneLiner.h"

/* 
 *  Constructs a ControlU which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
ControlU::ControlU( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : Control( parent, name, modal, fl ) {
data=seg=undoseg=NULL;
layerview=new SliceImage(NULL,"lv");
layerview->show();
connect(layerview,SIGNAL(doubleclicked(int,int,int)),this,SLOT(spotflood(int,int,int)));

alt->setMin(-90.0);
alt->setMax(0.0);
az->setMin(0);
az->setMax(360.0);

copyUPDOWN->setDisabled(TRUE);

QPopupMenu *m1;
m1=new QPopupMenu(NULL,"File");
menu->insertItem(QString("File"),m1,100);
m1->insertItem(QString("Open Model"),this,SLOT(Mreadimage()),0);
m1->insertItem(QString("Open Segment Map"),this,SLOT(Mreadseg()),0);
m1->insertItem(QString("Save Segment Map"),this,SLOT(Msaveseg()),0);
m1->insertSeparator();
m1->insertItem(QString("Exit"),this,SLOT(Mquit()),ALT+Key_X);

m1=new QPopupMenu(NULL,"Mask");
menu->insertItem(QString("Mask"),m1,100);
m1->insertItem(QString("Save Masks"),this,SLOT(Msavemask()),0);
m1->insertItem(QString("Save Masked Data"),this,SLOT(Msavemdata()),0);

layerd=layersegd=NULL;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ControlU::~ControlU() {
	delete layerview;
}

void ControlU::newshowseg(int i) {
layerview->setSegDisp(i);
}

void ControlU::newseg() {
QString s("Segment ");
s=s+QString::number(seglist->count());

seglist->insertItem(s,seglist->count());
showseg->setMaxValue(seglist->count()-1);
}

void ControlU::newlayer(int i) {
refreshlayer();
}

void ControlU::newrot(float f) {
refreshlayer();
}

void ControlU::newsegname() {
seglist->changeItem(segname->text(),seglist->currentItem());
}

void ControlU::newsegsel(int i) {
	layerview->setDrawSeg(i);
}

void ControlU::doUndo() {
refreshlayer();
if (!undoseg) return;
EMData *tmp = undoseg;
undoseg=seg;
seg=tmp;
refreshlayer();
}

void ControlU::copydown() {
Euler ort(alt->Value()*PI/180.0,0,az->Value()*PI/180.0);
layersegd->cutSlice(seg,layer->value()-1,&ort,0);
layersegd->unum2=1;
layerview->setSegData(layersegd);
}

void ControlU::copyup() {
Euler ort(alt->Value()*PI/180.0,0,az->Value()*PI/180.0);
layersegd->cutSlice(seg,layer->value()+1,&ort,0);
layersegd->unum2=1;
layerview->setSegData(layersegd);
}

void ControlU::copyupdown() {
    qWarning( "ControlU::copyupdown() not yet implemented!" ); 
}

void ControlU::newdrawrad() {
layerview->setDrawRad(drawrad->text().toInt());
}

void ControlU::newfillpix() {
}

void ControlU::newfillthr() {
}



void ControlU::spotflood(int x1, int y1, int segn) {
layersegd->uncutSlice(seg,(float)layersegd->unum,layersegd->getEuler());
layersegd->unum2=0;
float *mx = layersegd->getEuler()->mat();

if (undoseg) delete undoseg;
undoseg=seg->copy(0,0);

// Find the 3D coords of the double click position
float x2=x1-layersegd->xSize()/2;
float y2=y1-layersegd->ySize()/2;
int xx=round(x2*mx[0]+y2*mx[1]+layer->value()*mx[2]+seg->xSize()/2);
int yy=round(x2*mx[3]+y2*mx[4]+layer->value()*mx[5]+seg->ySize()/2);
int zz=round(x2*mx[6]+y2*mx[7]+layer->value()*mx[8]+seg->zSize()/2);

printf("Spotflood %d,%d,%d   %d\n",xx,yy,zz,segn);

seg->setValueAt(xx,yy,zz,-segn);

int iter=atoi((const char *)filliter->text());
float thr=fillthr->text().toFloat();

int i,ii,jj,kk,x,y,z;
for (i=1; i<iter; i++) {
	for (z=zz-i; z<=zz+i; z++) {
		for (y=yy-i; y<=yy+i; y++) {
			for (x=xx-i; x<=xx+i; x++) {
				if (x==xx-i+1 && y!=yy-i && y!=yy+i && z!=zz-i && z!=zz+i) x=xx+i;	// this makes a hollow cube
				if (x<1||y<1||z<1||x>seg->xSize()-2||y>seg->ySize()-2||z>seg->zSize()-2) continue;
				
				if (data->valueAt(x,y,z)<thr || (seg->valueAt(x,y,z)!=0 && seg->valueAt(x,y,z)!=segn)) continue;
				
				int g=0;
				for (kk=z-1; kk<=z+1; kk++) {
					for (jj=y-1; jj<=y+1; jj++) {
						for (ii=x-1; ii<=x+1; ii++) {
							if (seg->valueAt(ii,jj,kk)==-segn) g=1;
						}
					}
				}
				if (g) seg->setValueAt(x,y,z,-segn);
			}
		}
	}
}
seg->realFilter(4);
layersegd->unum2=0;
refreshlayer();
}

void ControlU::refreshlayer() {
if (!data || !seg) return;

if (!layerd || layerd->xSize()!=dim) {
	if (layerd) delete layerd;
	if (layersegd) delete layersegd;
	layerd=new EMData;
	layerd->setSize(dim,dim,1);
	layersegd=new EMData;
	layersegd->setSize(dim,dim,1);
}
else {
	if (layersegd->unum2) {
//		printf("%f,%f   %d %d",layersegd->alt(),layersegd->phi(),layersegd->unum,layersegd->unum2);
		if (undoseg) delete undoseg;
		undoseg=seg->copy(0,0);
		layersegd->uncutSlice(seg,layersegd->unum,layersegd->getEuler());
	}
}
Euler ort(alt->Value()*PI/180.0,0,az->Value()*PI/180.0);
layerd->cutSlice(data,layer->value(),&ort,1);
layersegd->cutSlice(seg,layer->value(),&ort,0);

layersegd->setRAlign(&ort);
layersegd->unum=layer->value();
layersegd->unum2=0;
layerview->setData(layerd);
layerview->setSegData(layersegd);

}

void ControlU::Mreadimage() {
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

if (!data) data=new EMData;
data->readImage((const char *)s,-1);

if (!seg) { seg=data->copyHead(); seg->zero(); }
if (data->xSize()!=seg->xSize() || data->ySize()!=seg->ySize() || data->zSize()!=seg->zSize()) {
	seg->setSize(data->xSize(),data->ySize(),data->zSize());
	seg->zero();
}
dim=data->xSize();
if (data->ySize()>dim) dim=data->ySize();
if (data->zSize()>dim) dim=data->zSize();

layer->setMinValue(-dim*17/20);
layer->setMaxValue(dim*17/20);

layerview->setMin(data->Min());
layerview->setMax(data->Max());

refreshlayer();
}

void ControlU::Mquit() {
exit(0);
}

void ControlU::doV5D() {
char s[256];

data->writeMRC("qseg1.mrc");
seg->writeMRC("qseg2.mrc");
system("mrc2v5d qseg1.mrc qseg2.mrc qseg.v5d");

//sprintf(s,"vis5d -box 1 1 1 -path / -script %s/doc/qsegscript.tcl qseg.v5d",EMDIR);
sprintf(s,"vis5d -box 1 1 1 -path / -mbs 384 qseg.v5d");

system(s); 
}

void ControlU::Mreadseg() {
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

if (!seg) seg=new EMData;
seg->readMRC((const char *)s);
if (data->xSize()!=seg->xSize() || data->ySize()!=seg->ySize() || data->zSize()!=seg->zSize()) {
	seg->setSize(data->xSize(),data->ySize(),data->zSize());
	seg->zero();
}

mrcH *hed=seg->getMRCHed();
seglist->clear();
seglist->insertItem(QString("Clear"));
for (int i=1; i<=seg->Max(); i++) {
	if (i<=10 && strncmp(hed->labels[i-1],"EMAN",4)!=0 && strlen(hed->labels[i-1])>1) {
		hed->labels[i-1][30]=0;
		seglist->insertItem(QString(hed->labels[i-1]));
	}
	else seglist->insertItem(QString("Segment ")+QString::number(seglist->count()));
}

layersegd->unum2=0;
refreshlayer();
}

void ControlU::Msaveseg() {
QString s( QFileDialog::getSaveFileName(0,"*.mrc") );
if ( s.isNull() ) return;

if (!seg->getMRCHed()) seg->writeMRC("/dev/null");
mrcH *hed=seg->getMRCHed();

for (int i=0; i<10; i++) {
	sprintf(hed->labels[i],"%0.79s",(const char *)seglist->text(i+1));
}

seg->writeMRC((const char *)s);
}

void ControlU::Msavemask() {
QString s=OneLiner::getOneLine("Enter filename prefix", "mask.");
int i;
if (s.length()<1) return;

for (i=0; i<=seg->Max(); i++) {
	EMData *cp=seg->copy(0,0);
	cp->realFilter(14,(float)i,(float)i);
	cp->writeMRC(s+QString::number(i)+QString(".mrc"));
	delete cp;
}

}

void ControlU::Msavemdata() {
QString s=OneLiner::getOneLine("Enter filename prefix", "masked.");
int i;
if (s.length()<1) return;

for (i=0; i<=seg->Max(); i++) {
	EMData *cp=seg->copy(0,0);
	cp->realFilter(14,(float)i,(float)i);
	cp->mult(data);
	cp->writeMRC(s+QString::number(i)+QString(".mrc"));
	delete cp;
}

}

EMData* ControlU::getSeg() { 
	if (!seg) {
		seg=data->copyHead();
		seg->zero();
	}
	return seg;
}
