// avg3d.C 11/99  Steve Ludtke
// Copyright 1999 Baylor College of Medicine

//N avg3d
//F This program will average a set of aligned 3d images together
//T LT
//1
//P [keep=<sigma mul>]	Optionally use only a subset of the images which are the most similar
//P <output file>	Output MRC file
//P <input 1>	First input
//P <input 2>	Second input
//P etc.
//U avg3d keep=.2 out.mrc in1.mrc in2.mrc in3.mrc in4.mrc
//D This averages a set of already aligned 3D images. It can optionally
//E use only the most self-similar subset of the input images. If keep
//E is specified, the images will be compared to the average, and only
//E those that match best will be used for the final average written
//E to the file. keep is specified as a multiplier on the standard
//E deviation of the the individual image matching factors. keep=0 will
//E use 1/2 of the images.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
main(int argc, char *argv[]) {
EMData *d1,*av;
int i,ff=1,kp=0,nk;
float keep=0,*dt;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);

if (strncmp(argv[1],"keep=",5)==0) {
	ff=2;
	kp=1;
	sscanf(&argv[1][5],"%f",&keep);
}

av=new EMData;
av->readImage(argv[ff+1],-1);
d1=new EMData;

for (i=ff+2; i<argc; i++) {
	d1->readImage(argv[i],-1);
	d1->normalize();
	av->add(d1);
}

av->normalize();

if (kp) {
	float mean=0,sigma=0,thr,dot;
	int n=argc-ff-1;
	dt=(float *)malloc(n*sizeof(float));
	
	for (i=ff+1; i<argc; i++) {
		d1->readImage(argv[i],-1);
		d1->normalize();
		dt[i-ff-1]=dot=d1->dot(av);
		mean+=dot;
		sigma+=dot*dot;
	}
	
	mean/=(float)n;
	sigma=sqrt(sigma/(float)n-mean*mean);
	thr=mean-sigma*keep;

	printf("mean=%1.1f  sigma=%1.1f  thr=%f\n",mean,sigma,thr);

	av->zero();
	nk=0;
	mean=sigma=0;
	for (i=ff+1; i<argc; i++) {
		d1->readImage(argv[i],-1);
		printf("%s\t%d. %f ",argv[i],i-ff,dt[i-ff-1]);
		if (dt[i-ff-1]<thr) { printf("\n"); continue; }
		printf("#\n");
		mean+=dt[i-ff-1];
		sigma+=dt[i-ff-1]*dt[i-ff-1];
		nk++;
		d1->normalize();
		av->add(d1);
	}

	av->normalize();

	mean/=(float)nk;
	sigma=sqrt(sigma/(float)nk-mean*mean);
	printf("mean=%1.1f  sigma=%1.1f  kept %0d/%0d\n",mean,sigma,nk,n);
}


av->writeMRC(argv[ff]);
}
